#include "DistrhoPlugin.hpp"
#include <array>

struct Voice
{
    double hz;
    double phase;
};

constexpr unsigned int NUM_VOICES = 256;
constexpr unsigned int VOICE_MASK = NUM_VOICES - 1;

class Synth
{
public:
    explicit Synth(double sampleRate);
    void process(float *output, uint32_t frames);
    void setPhaseVelocity(double in);
    void setTimbre(double in);
    void setSampleRate(double oldRate, double newRate);
    void setVolume(double in);
    void shiftUp();
    void shiftDown();

private:
    //Phase must persist between run invocation.
    //Phase of each voice, considered a function on the unit circle.
    std::array<double, NUM_VOICES> phases = {0};
    float volume = 0.0f;
    //Parameter in unit circle controlling pitch (varies by one octave).
    double spectrumPhase = 1.0f;
    //Revolutions per second of spectrum phase.
    double spectrumPhaseVelocity = 0.0;
    //Lowest fundamental frequency of blit.
    static inline constexpr double fMin = 55.0;
    //Current fundamental frequency of blit.
    double hzFund = fMin;
    double hzNyq = 24000.0;
    double sampleInterval = 1.0 / 48000.0;
};